<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace k2\com_k2\model\export;

defined('_JEXEC') or die;

use Joomla\CMS\Language\Text;
use Joomla\CMS\Filesystem\File;

require_once(JPATH_SITE.'/components/com_k2/helpers/route.php');

/**
 * Export K2 items.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Item extends \CsviModelExports
{
	/**
	 * Array of extra custom fields that can be used as available field.
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $customfieldsExport = [];

	/**
	 * Array of plugin fields that can be used as available field.
	 *
	 * @var    array
	 * @since  6.0
	 */
	private $pluginfieldsExport = [];

	/**
	 * The domain name for URLs.
	 *
	 * @var    string
	 * @since  6.0
	 */
	private $domainname = null;

	/**
	 * The addon helper
	 *
	 * @var    \Com_K2HelperCom_K2
	 * @since  6.0
	 */
	protected $helper = null;

	/**
	 * List of authors created articles
	 *
	 * @var    array
	 * @since  8.6.0
	 */
	private $cache = [];

	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$this->domainname = $this->settings->get('hostname');
			$this->loadExtraFields();
			$this->loadPluginFields();
			$userfields   = [];
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupFields   = $this->template->get('groupbyfields', [], 'array');
			$groupBy       = [];
			$groupByFields = [];

			if (!empty($groupFields))
			{
				foreach ($groupFields as $groupField)
				{
					$groupByFields[$groupField->name] = $groupField->name;
				}
			}

			// Sort selected fields
			$sortFields   = $this->template->get('sortfields', [], 'array');
			$sortBy       = [];
			$sortByFields = [];

			if (!empty($sortFields))
			{
				foreach ($sortFields as $sortName)
				{
					$sortByFields[$sortName->name] = $sortName->sortby;
				}
			}

			$userfields[] = $this->db->quoteName('k2_items.catid');
			$userfields[] = $this->db->quoteName('k2_items.alias');

			foreach ($exportfields as $field)
			{
				$sortDirection = ($sortByFields[$field->field_name]) ?? 'ASC';

				switch ($field->field_name)
				{
					case 'category_path':
						$userfields[] = $this->db->quoteName('k2_items.catid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'image':
					case 'image_url':
					case 'image_url_thumb':
						$userfields[] = "MD5(CONCAT('Image', " . $this->db->quoteName('k2_items.id') . ")) AS " . $this->db->quoteName('image');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'item_url':
					case 'tags':
					case 'id':
						$userfields[] = $this->db->quoteName('k2_items.id');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'metadata_author':
					case 'metadata_robot':
						$userfields[] = $this->db->quoteName('k2_items.metadata');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'hits':
						$userfields[] = $this->db->quoteName('k2_items.hits');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'title':
						$userfields[] = $this->db->quoteName('k2_items.title');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
						}
						break;
					case 'attachments_filename':
					case 'attachments_hits':
					case 'attachments_title':
					case 'attachments_titleAttribute':
						$userfields[] = $this->db->quoteName('k2_items.id');
						break;
					case 'created_by_name':
						$userfields[] = $this->db->quoteName('k2_items.created_by');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('k2_items.created_by');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('k2_items.created_by') . ' ' . $sortDirection;
						}
						break;
					case 'modified_by_name':
						$userfields[] = $this->db->quoteName('k2_items.modified_by');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('k2_items.modified_by');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('k2_items.modified_by') . ' ' . $sortDirection;
						}
						break;
					case 'checked_out_name':
						$userfields[] = $this->db->quoteName('k2_items.checked_out');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('k2_items.checked_out');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('k2_items.checked_out') . ' ' . $sortDirection;
						}
						break;
					case 'custom':
						break;
					default:
						if (in_array($field->field_name, $this->customfieldsExport))
						{
							$userfields[] = $this->db->quoteName('k2_items.catid');
							$userfields[] = $this->db->quoteName('extra_fields');

							if (array_key_exists($field->field_name, $groupByFields))
							{
								$groupBy[] = $this->db->quoteName('extra_fields');
							}

							if (array_key_exists($field->field_name, $sortByFields))
							{
								$sortBy[] = $this->db->quoteName('extra_fields') . ' ' . $sortDirection;
							}
						}
						elseif (in_array($field->field_name, $this->pluginfieldsExport))
						{
							$userfields[] = $this->db->quoteName('k2_items.id');
							$userfields[] = $this->db->quoteName('k2_items.plugins');

							if (array_key_exists($field->field_name, $groupByFields))
							{
								$groupBy[] = $this->db->quoteName('k2_items.plugins');
							}

							if (array_key_exists($field->field_name, $sortByFields))
							{
								$sortBy[] = $this->db->quoteName('k2_items.plugins') . ' ' . $sortDirection;
							}
						}
						else
						{
							$userfields[] = $this->db->quoteName($field->field_name);

							if (array_key_exists($field->field_name, $groupByFields))
							{
								$groupBy[] = $this->db->quoteName($field->field_name);
							}

							if (array_key_exists($field->field_name, $sortByFields))
							{
								$sortBy[] = $this->db->quoteName($field->field_name) . ' ' . $sortDirection;
							}
						}
						break;
				}
			}

			// Build the query
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__k2_items', 'k2_items'));

			// Filter by published state
			$publish_state = $this->template->get('publish_state');

			if ($publish_state != '' && ($publish_state == 1 || $publish_state == 0))
			{
				$query->where($this->db->quoteName('k2_items.published') . ' = ' . (int) $publish_state);
			}

			// Filter by language
			$language = $this->template->get('language');

			if ($language != '*')
			{
				$query->where($this->db->quoteName('k2_items.language') . ' = ' . $this->db->quote($language));
			}

			// Filter by category
			$categories = $this->template->get('item_categories');

			if ($categories && $categories[0] != '')
			{
				$query->where($this->db->quoteName('catid') . " IN ('" . implode("','", $categories) . "')");
			}

			// Filter by tags
			$tags = $this->template->get('item_tags');

			if ($tags && $tags[0] != '')
			{
				$query->innerJoin(
					$this->db->quoteName('#__k2_tags_xref', 'tx')
					. ' ON ' . $this->db->quoteName('tx.itemID') . ' = ' . $this->db->quoteName('k2_items.id') . ' AND ' .
					$this->db->quoteName('tx.tagID') . ' IN (' . implode(",", $tags) . ')'
				);
			}

			// Filter by author
			$authors = $this->template->get('item_authors');

			if ($authors && $authors[0] != '')
			{
				$query->where($this->db->quoteName('k2_items.created_by') . " IN ('" . implode("','", $authors) . "')");
			}

			// Filter by created start date
			$createdDateStart = $this->template->get('item_createdstart', false);

			if ($createdDateStart)
			{
				$createdDate = \JFactory::getDate($createdDateStart);
				$query->where($this->db->quoteName('k2_items.created') . ' >= ' . $this->db->quote($createdDate->toSql()));
			}
			// Filter by created end date
			$createdDateEnd = $this->template->get('item_createdend', false);

			if ($createdDateEnd)
			{
				$createdDate = \JFactory::getDate($createdDateEnd);
				$query->where($this->db->quoteName('k2_items.created') . ' <= ' . $this->db->quote($createdDate->toSql()));
			}
			// Filter by modified start date
			$modifiedDateStart = $this->template->get('item_modifiedstart', false);

			if ($modifiedDateStart)
			{
				$modifiedDate = \JFactory::getDate($modifiedDateStart);
				$query->where($this->db->quoteName('k2_items.modified') . ' >= ' . $this->db->quote($modifiedDate->toSql()));
			}
			// Filter by modified end date
			$modifiedDateEnd = $this->template->get('item_modifiedend', false);

			if ($modifiedDateEnd)
			{
				$modifiedDate = \JFactory::getDate($modifiedDateEnd);
				$query->where($this->db->quoteName('k2_items.modified') . ' <= ' . $this->db->quote($modifiedDate->toSql()));
			}

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (!empty($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (!empty($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$stm = $this->db->replacePrefix($query->__toString());

			if ((int) $limits['limit'] > 0 || (int) $limits['offset'] > 0)
			{
				$stm .= ' LIMIT ' . (int) $limits['offset'] . ', ' . (int) $limits['limit'];
			}

			$records = $this->pdo->yieldObjects($stm);
			$this->log->add('Export query' . $stm, false);

			// Check if there are any records
			$logcount = $this->pdo->fetchAffected($stm);

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportfields as $field)
					{
						$fieldname  = $field->field_name;
						$fieldvalue = '';

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}

						// Process the field
						switch ($fieldname)
						{
							case 'category_path':
								$fieldvalue = trim($this->helper->createCategoryPathById($record->catid));
								break;
							case 'item_url':
								// Let's create a SEF URL
								$categoryAlias = $this->helper->getCategoryAlias($record->catid);
								$routerK2      = new \K2HelperRoute();
								$fieldvalue    = $this->sef->getSefUrl(urldecode($routerK2->getItemRoute($record->id . ':' . urlencode($record->alias), $record->catid . ':' . urlencode($categoryAlias))));
								break;
							case 'image':
								$fieldvalue = '';

								// Check if there is already a product full image
								if (File::exists(JPATH_SITE . '/media/k2/items/cache/' . $record->image . '.jpg') ||
									File::exists(JPATH_SITE . '/media/k2/items/src/' . $record->image . '.jpg'))
								{
									$fieldvalue = $record->image . '.jpg';
								}
								break;
							case 'image_url':
								// Check if there is already a product full image
								if (File::exists(JPATH_SITE . '/media/k2/items/cache/' . $record->image . '_' . $this->template->get('image_url', 'Generic') . '.jpg')
									|| File::exists(JPATH_SITE . '/media/k2/items/src/' . $record->image . '.jpg'))
								{
									$fieldvalue = $this->domainname . '/media/k2/items/cache/' . $record->image . '_' . $this->template->get('image_url', 'Generic') . '.jpg';
								}
								break;
							case 'image_url_thumb':
								// Check if there is already a product full image
								if (File::exists(JPATH_SITE . '/media/k2/items/cache/' . $record->image . '_' . $this->template->get('image_url_thumb', 'S') . '.jpg')
									|| File::exists(JPATH_SITE . '/media/k2/items/src/' . $record->image . '.jpg'))
								{
									$fieldvalue = $this->domainname . '/media/k2/items/cache/' . $record->image . '_' . $this->template->get('image_url_thumb', 'S') . '.jpg';
								}
								break;
							case 'metadata_author':
							case 'metadata_robot':
								$metadata = explode("\n", $record->metadata);

								foreach ($metadata as $metafield)
								{
									list($name, $value) = explode('=', $metafield);

									switch ($fieldname)
									{
										case 'metadata_robot':
											if ($name == 'robots')
											{
												$fieldvalue = $value;
											}
											break;
										case 'metadata_author':
											if ($name == 'author')
											{
												$fieldvalue = $value;
											}
											break;
									}
								}
								break;
							case 'tags':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('t.name'))
									->from($this->db->quoteName('#__k2_tags_xref', 'x'))
									->leftJoin(
										$this->db->quoteName('#__k2_tags', 't')
										. ' ON ' . $this->db->quoteName('t.id') . ' = ' . $this->db->quoteName('x.tagID')
									)
									->where($this->db->quoteName('itemID') . ' = ' . (int) $record->id);
								$this->db->setQuery($query);
								$fieldvalue = implode('|', $this->db->loadColumn());
								break;
							case 'attachments_title':
							case 'attachments_hits':
							case 'attachments_filename':
							case 'attachments_titleAttribute':
								$fieldname = str_replace('attachments_', '', $fieldname);

								$query = $this->db->getQuery(true)
									->select($this->db->quoteName($fieldname))
									->from($this->db->quoteName('#__k2_attachments'))
									->where($this->db->quoteName('itemID') . ' = ' . (int) $record->id);
								$this->db->setQuery($query);
								$fieldvalue = implode('|', $this->db->loadColumn());
								break;
							case 'created_by_name':
								if (!array_key_exists($fieldname . $record->created_by, $this->cache))
								{
									$query->clear()
										->select($this->db->quoteName('name'))
										->from($this->db->quoteName('#__users'))
										->where($this->db->quoteName('id') . ' = ' . (int) $record->created_by);
									$this->db->setQuery($query);
									$this->cache[$fieldname . $record->created_by] = $this->db->loadResult();
								}

								$fieldvalue = $this->cache[$fieldname . $record->created_by];
								break;
							case 'modified_by_name':
								if (!array_key_exists($fieldname . $record->modified_by, $this->cache))
								{
									$query->clear()
										->select($this->db->quoteName('name'))
										->from($this->db->quoteName('#__users'))
										->where($this->db->quoteName('id') . ' = ' . (int) $record->modified_by);
									$this->db->setQuery($query);
									$this->cache[$fieldname . $record->modified_by] = $this->db->loadResult();
								}

								$fieldvalue = $this->cache[$fieldname . $record->modified_by];
								break;
							case 'checked_out_name':
								if (!array_key_exists($fieldname . $record->checked_out, $this->cache))
								{
									$query->clear()
										->select($this->db->quoteName('name'))
										->from($this->db->quoteName('#__users'))
										->where($this->db->quoteName('id') . ' = ' . (int) $record->checked_out);
									$this->db->setQuery($query);
									$this->cache[$fieldname . $record->checked_out] = $this->db->loadResult();
								}

								$fieldvalue = $this->cache[$fieldname . $record->checked_out];
								break;
							default:
								if (in_array($fieldname, $this->customfieldsExport))
								{
									// Get the extra fields values
									$record_extrafields = json_decode($record->extra_fields);
									$groupId            = $this->helper->getExtraFieldGroup($record->catid);
									$groupName          = $this->helper->getExtraFieldGroupName($groupId);

									if (is_array($record_extrafields))
									{
										// Extract the title from the field
										$separatorPosition = stripos($fieldname, '_');
										$title = substr($fieldname, $separatorPosition + 1);

										$query = $this->db->getQuery(true)
											->select($this->db->quoteName(['id', 'type', 'value']))
											->from($this->db->quoteName('#__k2_extra_fields'))
											->where($this->db->quoteName('name') . ' = ' . $this->db->quote($title))
											->where($this->db->quoteName('group') . ' = ' . $groupId);
										$this->db->setQuery($query);
										$extra        = $this->db->loadObject();
										$extra->value = json_decode($extra->value);

										if (is_object($extra))
										{
											// Check if this item has this field
											foreach ($record_extrafields as $extrafield)
											{
												if ($extrafield->id == $extra->id && (strpos($fieldname, $groupName) !== false))
												{
													switch ($extra->type)
													{
														case 'multipleSelect':
															// Multiple selects are | separated
															$values = [];

															foreach ($extra->value as $extra_value)
															{
																if (in_array($extra_value->value, $extrafield->value))
																{
																	$values[] = $extra_value->name;
																}
															}

															$fieldvalue = implode('|', $values);
															break;
														case 'radio':
														case 'select':
															foreach ($extra->value as $extra_value)
															{
																if ($extra_value->value == $extrafield->value)
																{
																	$fieldvalue = $extra_value->name;
																}
															}
															break;
														case 'link':
															$fieldvalue = implode('|', $extrafield->value);
															break;
														default:
															$fieldvalue = $extrafield->value;
															break;
													}
												}
											}
										}
									}
								}
								elseif (in_array($fieldname, $this->pluginfieldsExport))
								{
									// Get value from K2 plugin
									$dispatcher = new \RantaiPluginDispatcher;
									$dispatcher->importPlugins('csviext', $this->db);
									$result = $dispatcher->trigger(
										'onExportContent',
										[
											'extension' => 'k2',
											'id' => $record->id,
											'fieldname' => $fieldname,
											'operation' => 'item',
											'log' => $this->log
										]
									);

									$fieldvalue = $result[0];
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(Text::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}

	/**
	 * Get a list of extra fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   4.4.1
	 */
	private function loadExtraFields()
	{
		$query = $this->db->getQuery(true)
			->select('TRIM(
						CONCAT(' .
				$this->db->quoteName('g.name') . ', ' . $this->db->quote('_') . ', ' . $this->db->quoteName('x.name') .
				') ' .
				') AS ' . $this->db->quoteName('title')
			)
			->from($this->db->quoteName('#__k2_extra_fields', 'x'))
			->leftJoin(
				$this->db->quoteName('#__k2_extra_fields_groups', 'g')
				. ' ON ' . $this->db->quoteName('g.id') . ' = ' . $this->db->quoteName('x.group')
			);
		$this->db->setQuery($query);
		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = [];
		}

		$this->customfieldsExport = $result;
	}

	/**
	 * Get a list of plugin fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   6.5.0
	 */
	private function loadPluginFields()
	{
		$dispatcher = new \RantaiPluginDispatcher;
		$dispatcher->importPlugins('csviext', $this->db);
		$result = $dispatcher->trigger(
			'getAttributes',
			[
				'extension' => 'k2',
				'operation' => 'item',
				'log' => $this->log
			]
		);

		if (is_array($result) && !empty($result))
		{
			$this->pluginfieldsExport = array_merge($this->pluginfieldsExport, $result[0]);
		}
	}
}
